@@ -18,14 +18,30 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #---------------------------------------------------------------------------
+import shutil
 from base_module import ranaModule
+import glob
 import time
 import os
+from collections import deque
 import geo
+import way
 from core import gs
+
 if gs.GUIString == "GTK":
   import gtk
 
+DONT_ADD_TO_TRACE_THRESHOLD = 1
+# if a point is less distant from the last
+# point added to the trace than DONT_ADD_TO_TRACE_THRESHOLD
+# it is not added to the trace point list
+#
+# Usage: to get rid of points that have very similar coordinates,
+# that are generated when for example waiting
+# at teh traffic lights
+#
+# NOTE: all points are still stored, just not drawn
+
 
 def getModule(m,d,i):
   return(tracklog(m,d,i))
@@ -40,14 +56,19 @@ class tracklog(ranaModule):
     self.loggingStartTimestamp = None
     self.logInterval = 1 #loggin interval in seconds
     self.saveInterval = 10 #saving interval in seconds
-    self.lastTimestamp = None
-    self.lastSavedTimestamp = None
+    self.lastUpdateTimestamp = None
     self.lastCoords = None
-    self.currentLogGPX = None #a GPX tree for current log
-    self.currentTempLog = None #a temporary log segment before saving to file
-    self.currentLogName = None #name of the current log
-    self.currentLogFileName = None #name of the current log
-    self.currentLogPath = None #path to the current log
+    self.logName = None #name of the current log
+    self.logFilename = None #name of the current log
+    self.logPath = None #path to the current log
+    # primary and secondary AOWay objects for
+    # persistent log storage during logging
+    self.log1 = None
+    self.log2 = None
+    # timer ids
+    self.updateLogTimerId = None
+    self.saveLogTimerId = None
+    # statistics
     self.maxSpeed = None
     self.avg1 = 0
     self.avg2 = 0
@@ -55,53 +76,48 @@ class tracklog(ranaModule):
     self.distance = None
     self.toolsMenuDone = False
     self.category='logs'
+    # trace
     self.traceColor = 'blue'
+    self.lastTracePoint = None
     self.traceIndex = 0
-    self.pxpyIndex = []
+    self.pxpyIndex = deque()
 
-#    self.startupTimestamp = time.strftime("%Y%m%dT%H%M%S")
+  #    self.startupTimestamp = time.strftime("%Y%m%dT%H%M%S")
 
+  def firstTime(self):
+    # rescue any log files that were not exported to GPX previously
+    # eq, due to modRana or device crashing
+    self._rescueLogs()
 
   def handleMessage(self, message, type, args):
     if message == "incrementStartIndex":
       self.startButtonIndex = (self.startButtonIndex+1)%2 # when we go to 2, we return to 0
     elif message == "startLogging":
-      print "starting to log"
+      print("tracklog: starting to log")
       # start a new log
       if not self.loggingEnabled:
         self.loggingEnabled = True
-        print "initializing the log file"
+        print("tracklog: initializing the log file")
         self.initLog()
       # or resume an existing one
       elif self.loggingEnabled == True & self.loggingPaused == True:
-        print "resuming the logging"
+        print "tracklog: resuming the logging"
         self.loggingPaused = False
       self.set('needRedraw', True)
 
     elif message == "pauseLogging":
-      print "pausing the logging"
-      self.saveLogIncrement()
-      self.loggingPaused = True
+      self.pauseLogging()
       self.set('needRedraw', True)
 
     elif message == "stopLogging":
-      print "stopping the logging"
+      print("tracklog: stopping logging")
       self.stopLogging()
       self.set('needRedraw', True)
-      
-    elif message == "setNewLoggingInterval":
-      print "setting new log interval"
-      self.logInterval=int(self.get('tracklogLogInterval', 1))
-      self.set('needRedraw', True)
-
-    elif message == "setNewSavingInterval":
-      print "setting new save interval"
-      self.saveInterval=int(self.get('tracklogSaveInterval', 10))
-      self.set('needRedraw', True)
 
     elif message == 'nameInput':
       entry = self.m.get('textEntry', None)
       if entry is None:
+        print("tracklog: error, text entry module is not loaded")
         return
       entryText = ""
       logNameEntry = self.get('logNameEntry', None)
@@ -127,42 +143,42 @@ class tracklog(ranaModule):
     if key == 'logNameEntry':
       self.set('logNameEntry', result)
 
-  def update(self):
-    if self.loggingEnabled & (not self.loggingPaused):
-      currentTimestamp = time.time()
-
-      # update the current log speed statistics
-      currentSpeed = self.get('speed', None)
-      if currentSpeed:
-        # max speed
-        if currentSpeed>self.maxSpeed:
-          self.maxSpeed = currentSpeed
-        # avg speed
-        self.avg1 += currentSpeed
-        self.avg2 += (currentTimestamp - self.lastTimestamp)
-        self.avgSpeed = self.avg1/self.avg2
-
-      if (currentTimestamp - self.lastTimestamp)>self.logInterval:
-        print "updating the log"
-        (lat,lon) = self.get('pos', None)
-        elevation = self.get('elevation', None) # TODO: add elevation logging support
-        currentLatLonElevTime = (lat,lon, elevation, time.strftime("%Y-%m-%dT%H:%M:%S"))
-        self.currentTempLog.append(currentLatLonElevTime)
-        self.lastTimestamp = currentTimestamp
-        # update traveled distance
-        self.storeCurrentPosition()
-        currentCoords = self.get('pos', None)
-        if self.lastCoords is not None and currentCoords is not None:
-          (lat1,lon1) = self.lastCoords
-          (lat2,lon2) = currentCoords
-          self.distance+=geo.distance(lat1,lon1,lat2,lon2)
-          self.lastCoords = currentCoords
-
-      if (currentTimestamp - self.lastSavedTimestamp)>self.saveInterval:
-        print "saving log increment"
-        self.saveLogIncrement()
-        self.lastSavedTimestamp = currentTimestamp
-        self.set('needRedraw', True)
+#  def update(self):
+#    if self.loggingEnabled & (not self.loggingPaused):
+#      currentTimestamp = time.time()
+#
+#      # update the current log speed statistics
+#      currentSpeed = self.get('speed', None)
+#      if currentSpeed:
+#        # max speed
+#        if currentSpeed>self.maxSpeed:
+#          self.maxSpeed = currentSpeed
+#        # avg speed
+#        self.avg1 += currentSpeed
+#        self.avg2 += (currentTimestamp - self.lastTimestamp)
+#        self.avgSpeed = self.avg1/self.avg2
+#
+#      if (currentTimestamp - self.lastTimestamp)>self.logInterval:
+#        print("tracklog: updating the log")
+#        (lat,lon) = self.get('pos', None)
+#        elevation = self.get('elevation', None) # TODO: add elevation logging support
+#        currentLatLonElevTime = (lat,lon, elevation, time.strftime("%Y-%m-%dT%H:%M:%S"))
+#        self.currentTempLog.append(currentLatLonElevTime)
+#        self.lastTimestamp = currentTimestamp
+#        # update traveled distance
+#        self.storeCurrentPosition()
+#        currentCoords = self.get('pos', None)
+#        if self.lastCoords is not None and currentCoords is not None:
+#          (lat1,lon1) = self.lastCoords
+#          (lat2,lon2) = currentCoords
+#          self.distance+=geo.distance(lat1,lon1,lat2,lon2)
+#          self.lastCoords = currentCoords
+#
+#      if (currentTimestamp - self.lastSavedTimestamp)>self.saveInterval:
+#        print("tracklog: saving log increment")
+#        self._saveLogIncrement()
+#        self.lastSavedTimestamp = currentTimestamp
+#        self.set('needRedraw', True)
 
   def initLog(self,type='gpx',name=None):
     """start a new log, zero the appropriate variables, etc."""
@@ -173,14 +189,13 @@ class tracklog(ranaModule):
     self.avgSpeed = 0
     self.currentTempLog = []
     self.distance = 0
-    self.pxpyIndex = []
-    options = self.m.get('options', None)
-    tracklogFolder = self.modrana.paths.getTracklogsFolderPath()
+    self.pxpyIndex.clear()
+    logFolder = self.getLogFolderPath()
 
     if name is None:
       name = self.generateLogName()
 
-    self.currentLogName = name
+    self.logName = name
 
     if type=='gpx':
       """ importing the GPX module can be time consuming so import it
@@ -194,48 +209,118 @@ class tracklog(ranaModule):
 #      self.currentLogGPX.author = "modRana - a flexible GPS navigation system"
 #      self.currentLogGPX.link = "http://nlp.fi.muni.cz/trac/gps_navigace"
 
-      filename = name + ".gpx"
-      self.currentLogFileName = filename
-      self.currentLogPath = os.path.join(tracklogFolder, self.category, filename)
-      self.saveGPXLog(self.currentLogGPX, self.currentLogFileName)
+      filename = "%s.gpx" % name
+      self.logFilename = filename
+      self.logPath = os.path.join(logFolder, filename)
 
-    self.lastTimestamp = self.lastSavedTimestamp = int(time.time())
+      # initialize temporary CSV log files
+      path1 = os.path.join(logFolder, "%s.temporary_csv_1" % name)
+      path2 = os.path.join(logFolder, "%s.temporary_csv_2" % name)
+
+      log1 = way.AppendOnlyWay()
+      log1.startWritingCSV(path1)
+      self.log1 = log1
+
+      log2 = way.AppendOnlyWay()
+      log2.startWritingCSV(path2)
+      self.log2 = log2
+
+      # start update and save timers
+      self._startTimers()
+
+    self.lastUpdateTimestamp = time.time()
     self.lastCoords = self.get('pos', None)
-    print "log file initialized"
+    print("tracklog: log file initialized")
 
-
-  def saveGPXLog(self, GPXTracklog, filename):
-#      f = open(self.currentLogPath,'w')
-#      xmlTree = self.currentLogGPX.export_gpx_file("1.1")
-#      xmlTree.write(f)
-#      f.close()
-    loadTl = self.m.get('loadTracklogs', None)
-    if loadTl:
-      loadTl.storeTracklog(GPXTracklog, filename, self.category, "GPX", False)
-
-
-
-  def saveLogIncrement(self):
-    """save current log increment
-    TODO: support for more log types"""
-    """
-    GPX Trackpoint takes parameters in this order:
-    latitude, longitude, name=None, description=None, elevation=None, time=None
-
-    """
-
-    """ importing the GPX module can be time consuming so import it
-    when it is really needed"""
-    from upoints import gpx
-    newTrackpoints = map(lambda x: gpx.Trackpoint(x[0],x[1],None,None,x[2],x[3]), self.currentTempLog)
-
-    if len(self.currentLogGPX)==0:
-      self.currentLogGPX.append(newTrackpoints)
+  def pauseLogging(self):
+    """pause logging"""
+    if self.loggingEnabled:
+      self._saveLogIncrement() # save increment
+      self.loggingPaused = True # pause logging
+      print('tracklog: logging paused')
     else:
-      self.currentLogGPX[0].extend(newTrackpoints)
-    self.saveGPXLog(self.currentLogGPX, self.currentLogFileName)
-    #the current temporary log segment has been saved to disk, we can empty it
-    self.currentTempLog = []
+      print("tracklog: can't pause logging - no logging in progress")
+
+  def unPauseLogging(self):
+    """pause logging"""
+    if self.loggingEnabled:
+      self._saveLogIncrement() # save increment
+      self.loggingPaused = False # un-pause logging
+      print('tracklog: logging un-paused')
+    else:
+      print("tracklog: can't un-pause logging - no logging in progress")
+
+  def _updateLogCB(self):
+    """add current position at the end of the log"""
+    pos = self.get('pos', None)
+    if pos and not self.loggingPaused:
+      timestamp = geo.timestampUTC()
+      lat, lon = pos
+      elevation = self.get('elevation', None)
+      self.log1.addPointLLET(lat, lon, elevation, timestamp)
+      self.log2.addPointLLET(lat, lon, elevation, timestamp)
+
+      # update statistics for the current log
+      if self.loggingEnabled and not self.loggingPaused:
+        # update the current log speed statistics
+        currentSpeed = self.get('speed', None)
+        if currentSpeed:
+          # max speed
+          if currentSpeed > self.maxSpeed:
+            self.maxSpeed = currentSpeed
+          # avg speed
+          self.avg1 += currentSpeed
+          self.avg2 += (time.time() - self.lastUpdateTimestamp)
+          self.avgSpeed = self.avg1/self.avg2
+          self.lastUpdateTimestamp = time.time()
+
+        # update traveled distance
+        if self.lastCoords:
+          lLat, lLon = self.lastCoords
+          self.distance+=geo.distance(lLat, lLon, lat, lon)
+          self.lastCoords = lat, lon
+
+        # update the on-map trace
+        if self.lastTracePoint:
+          lat1, lon1 = self.lastTracePoint
+          # check if the point is distant enough from the last added point
+          # (which was either the first point or also passed the test)
+          if geo.distanceApprox(lat, lon, lat1, lon1)*1000 >= DONT_ADD_TO_TRACE_THRESHOLD:
+            self._addLL2Trace(lat, lon)
+        else: # this is the first known log point, just add it
+          self._addLL2Trace(lat, lon)
+
+  def _addLL2Trace(self, lat, lon):
+    proj = self.m.get('projection')
+    if proj:
+      (px,py) = proj.ll2pxpyRel(lat, lon)
+      index = self.traceIndex
+      # the pxpyIndex deque is created in reverse order
+      # so that it doesn't need to be reversed when its
+      # drawn on the map from current position to first point
+      self.pxpyIndex.appendleft((px,py,index))
+      self.traceIndex+=1
+      self.lastTracePoint = (lat, lon)
+
+  def _saveLogCB(self):
+    """save the log to temporary files in storage
+    (only the increment from last save needs to be stored)"""
+    if not self.loggingPaused:
+      self._saveLogIncrement()
+      print('tracklog: temporary log files saved')
+
+  def _saveLogIncrement(self):
+    """save current log increment to storage"""
+    try:
+      self.log1.flush()
+    except Exception, e:
+      print('tracklog: saving primary temporary log failed')
+      print e
+    try:
+      self.log2.flush()
+    except Exception, e:
+      print('tracklog: saving secondary temporary log failed')
+      print e
 
   def generateLogName(self):
     """generate a unique name for a log"""
@@ -246,58 +331,102 @@ class tracklog(ranaModule):
       prefix = logNameEntry
     return prefix + "_" + timeString
 
+  def getLogFolderPath(self):
+    """return path to the log folder"""
+    tracklogFolder = self.modrana.paths.getTracklogsFolderPath()
+    return os.path.join(tracklogFolder, self.category)
+
+  def _startTimers(self):
+    """start the update and save timers"""
+    cron = self.m.get('cron', None)
+    if cron:
+      # in milliseconds, stored as seconds
+      updateTimeout = int(self.get('tracklogLogInterval', 1))*1000
+      saveTimeout = int(self.get('tracklogSaveInterval', 10))*1000
+      # update timer
+      self.updateLogTimerId = cron.addTimeout(self._updateLogCB, updateTimeout, self, "update tracklog with current position")
+      # save timer
+      self.saveLogTimerId = cron.addTimeout(self._saveLogCB, saveTimeout, self, "save tracklog increment")
+      # update timer intervals if they are changed
+      # in the persistent dictionary
+      self.modrana.watch('tracklogLogInterval', self._updateIntervalChangedCB)
+      self.modrana.watch('tracklogSaveInterval', self._saveIntervalChangedCB)
+
+  def _updateIntervalChangedCB(self, key, oldInterval, newInterval):
+    if self.updateLogTimerId:
+      cron = self.m.get('cron', True)
+      interval = int(newInterval)*1000
+      if cron:
+        cron.modifyTimeout(self.updateLogTimerId, interval)
+        print('tracklog: tracklog update interval changed to %s s' % newInterval)
+      else:
+        print("tracklog: error, the cron module is not loaded")
+
+  def _saveIntervalChangedCB(self, key, oldInterval, newInterval):
+    if self.updateLogTimerId:
+      cron = self.m.get('cron', True)
+      interval = int(newInterval)*1000
+      if cron:
+        cron.modifyTimeout(self.saveLogTimerId, interval)
+        print('tracklog: tracklog save interval changed to %s s' % newInterval)
+      else:
+        print("tracklog: error, the cron module is not loaded")
+
+  def _stopTimers(self):
+    """stop the update and save timers"""
+    cron = self.m.get('cron', None)
+    if cron:
+      cron.removeTimeout(self.updateLogTimerId)
+      cron.removeTimeout(self.saveLogTimerId)
+      self.saveLogTimerId = None
+      self.updateLogTimerId = None
+
   def stopLogging(self):
-      self.saveLogIncrement()
-#      path = self.currentLogPath
-      self.clean()
-      self.loggingEnabled = False
-      self.startButtonIndex=0
-      # now we make the tracklog manager aware, that there is a new log
-      loadTl = self.m.get('loadTracklogs', None)
-      if loadTl:
-#        # we also set the correct category ('log')
-#        loadTl.setTracklogPathCategory(path, 'log')
-        loadTl.listAvailableTracklogs() #TODO: incremental addition
+    """stop logging, export the log to GPX and delete the temporary
+    log files"""
+    # stop timers
+    self._stopTimers()
+
+    # save current log increment to storage (in CSV)
+    self._saveLogIncrement()
+
+    # try to export the log to GPX
+    self.notify("saving tracklog", 3000)
+    # first from the primary log
+    if not self.log1.saveToGPX(self.logPath):
+      self.log2.saveToGPX(self.logPath) # try the secondary log
+    # TODO: check if the GPX file is loadable and retry ?
+
+    # cleanup
+    # -> this deletes the temporary log files
+    # and discards the temporary AOWay objects
+    self._cleanup()
+    self.loggingEnabled = False
+    self.startButtonIndex=0
+    # now we make the tracklog manager aware, that there is a new log
+    loadTl = self.m.get('loadTracklogs', None)
+    if loadTl:
+      loadTl.listAvailableTracklogs() #TODO: incremental addition
 
 
-
-  def clean(self):
+  def _cleanup(self, deleteTempLogs=True):
     """zero unneeded datastructures after logging is stopped"""
-    self.currentLogGPX = None
-    self.currentTempLog = None
+
+    # delete the temporary log files
+    if deleteTempLogs:
+      self.log1.deleteFile()
+      self.log2.deleteFile()
+    self.log1 = None
+    self.log2 = None
+
+    # statistics
     self.loggingStartTimestamp = None
-#    self.maxSpeed = None
-#    self.avgSpeed = None
-
-
-  def storeCurrentPosition(self):
-    pos = self.get('pos', None)
-    proj =self.m.get('projection', None)
-
-    if pos and proj:
-      (lat,lon) = pos
-      (px,py) = proj.ll2pxpyRel(lat, lon)
-      index = self.traceIndex
-      """
-      for not, latLonIndex is unsorted, if we sort it according
-      to lat + lon + index to speed up filter, we would need another list in
-      insert order
-      """
-      self.pxpyIndex.append((px,py,index))
-      self.traceIndex+=1
-
-#    proj = self.m.get('projection', None)
-#    if proj:
-##      (lat,lon) = self.get('pos', None)
-#      (lat,lon) = (48.0,16.0)
-##      print proj.ll2pxpy(lat, lon)
-#      print (lat,lon)
-#      (px,py) =  proj.ll2pxpy(lat, lon)
-#      print (px, py)
-#      (x,y) = proj.pxpy2xy(px, py)
-#      print (x,y)
-#      print proj.xy2ll(x, y)
-
+    self.maxSpeed = None
+    self.avgSpeed = None
+    # on-map trace
+    self.lastTracePoint = None
+    self.traceIndex = 0
+    self.pxpyIndex.clear()
 
 #  def getVisiblePoints(self):
 #    # get only the points, that are currently visible
@@ -317,67 +446,6 @@ class tracklog(ranaModule):
 #      print visiblePoints
 #      return visiblePoints
 
-
-#  def load(self, filename):
-#    # TODO: share this with replayGpx
-#    self.nodes = []
-#    file = open(filename, 'r')
-#    if(file):
-#      """ importing the GPX module can be time consuming so import it
-#      when it is really needed"""
-#      from upoints import gpx
-#      track = gpx.Trackpoints() # create new Trackpoints object
-#      track.import_locations(file) # load a gpx file into it
-#      for point in track[0]: #iterate over the points, track[0] is list of all points in file
-#        lat = point.latitude
-#        lon = point.longitude
-#        self.nodes.append([lat,lon])
-#      file.close()
-#      self.numNodes = len(self.nodes)
-#      self.set("centreOnce", True)
-#      self.pos = int(self.get("replayStart",0) * self.numNodes)
-      
-#only for GPX 1.0, the above works for GPX 1.1
-#      regexp = re.compile("<trkpt lat=['\"](.*?)['\"] lon=['\"](.*?)['\"]")
-#      for text in file:
-#        matches = regexp.match(text)
-#        if(matches):
-#          lat = float(matches.group(1))
-#          lon = float(matches.group(2))
-#          self.nodes.append([lat,lon])
-#      file.close()
-
-#  def scheduledUpdate(self):
-#    pos = self.get('pos', None)
-#    if(pos != None):
-#      self.nodes.append(pos)
-#      #(lat,lon) = pos
-#      #print "Logging %f, %f" % (lat,lon)
-#    self.saveMinimal("data/tracklogs/%s.txt" % self.startupTimestamp);
-#
-#  def drawMapOverlay(self, cr):
-#    # Where is the map?
-#    proj = self.m.get('projection', None)
-#    if(proj == None):
-#      return
-#    if(not proj.isValid()):
-#      return
-#
-#    # Draw all trackpoints as lines (TODO: optimisation)
-#    cr.set_source_rgb(0,0,0.5)
-#    first = True
-#    for n in self.nodes:
-#      (lat,lon) = n
-#      (x,y) = proj.ll2xy(lat,lon)
-#      if(proj.onscreen(x,y)):
-#        if first: #the first coordinates are wrong co we skip them
-#          self.line(cr, x, y)
-#          first = False
-#        else:
-#          self.line(cr, x, y)
-#
-#      cr.fill()
-    
   def point(self, cr, x, y):
     s = 2 #default 2
     cr.rectangle(x-s,y-s,2*s,2*s)
@@ -463,7 +531,7 @@ class tracklog(ranaModule):
     if menuName == 'tracklog':
       # is the submenu initialized ?
       if self.toolsMenuDone == False:
-        print "setting up tracklogTools menu"
+        print("tracklog: setting up tracklogTools menu")
         self.initToolsMenu()
         self.toolsMenuDone = True
       # setup the viewport
@@ -508,7 +576,7 @@ class tracklog(ranaModule):
       if not self.loggingEnabled:
         text+= "%s" % self.generateLogName()
       else:
-        text+= "%s" % self.currentLogName
+        text+= "%s" % self.logName
 
       text+= "\n\nlogging interval %d s, saving every %d s" % (self.logInterval, self.saveInterval)
       if self.loggingStartTimestamp:
@@ -549,7 +617,6 @@ class tracklog(ranaModule):
 
   def drawMapOverlay(self, cr):
     proj = self.m.get('projection', None)
-
     if proj and self.pxpyIndex:
 #      cr.set_source_rgba(0, 0, 1, 1)
       cr.set_source_color(gtk.gdk.color_parse(self.traceColor))
@@ -562,12 +629,11 @@ class tracklog(ranaModule):
       TODO: use the modulo method for drawing stored tracklogs
       """
 
-      posXY = proj.getCurrentPospxpy()
+      posXY = proj.getCurrentPosXY()
       if posXY and self.loggingEnabled and not self.loggingPaused:
-        (x,y) = posXY
-        cr.move_to(x,y)
+        cr.move_to(*posXY) # start drawing from current position
       else:
-        (px,py,index) = self.pxpyIndex[-1]
+        (px,py,index) = self.pxpyIndex[0] # start drawing from first trace point
         (x,y) = proj.pxpyRel2xy(px, py)
         cr.move_to(x,y)
 
@@ -584,8 +650,8 @@ class tracklog(ranaModule):
       drawCount = 0
       counter=0
 #
-#
-      for point in reversed(self.pxpyIndex): #draw the track
+      #draw the track
+      for point in self.pxpyIndex:  # pypyIndex is already in reverse order
         counter+=1
         if counter%modulo==0:
           drawCount+=1
@@ -594,6 +660,7 @@ class tracklog(ranaModule):
           (px,py,index) = point
           (x,y) = proj.pxpyRel2xy(px, py)
           cr.line_to(x,y)
+      # TODO: don't iterate, just get items based on index
           
       # draw a track to current position (if known):
 
@@ -603,9 +670,103 @@ class tracklog(ranaModule):
       cr.stroke()
       cr.fill()
 
+  def _rescueLogs(self):
+    """rescue any log files that were not exported to GPX previously"""
+
+    # get log folder path
+    logFolder = self.getLogFolderPath()
+
+    # check out the log folder for temporary files
+
+    # first scan for primary logs
+    primaryLogs = glob.glob("%s/*.temporary_csv_1" % logFolder)
+    secondaryLogs = glob.glob("%s/*.temporary_csv_2" % logFolder)
+
+    if primaryLogs or secondaryLogs:
+      self.notify("exporting temporary tracklogs to GPX", 5000)
+      self.set('needRedraw', True)
+
+    if primaryLogs:
+      print('tracklog: exporting %d unsaved primary log files to GPX' % len(primaryLogs))
+      for logPath in primaryLogs:
+        # export any found files
+        print('tracklog: exporting %s to GPX' % logPath)
+        try:
+          w1 = way.fromCSV(logPath, delimiter=",")
+          exportPath = "%s.gpx" % os.path.splitext(logPath)[0]
+          # does the GPX file already exist ?
+          # TODO: check if the GPX file is corrupted and swap with newly exported one ?
+          # (eq. caused by a crash during saving the GPX file)
+          if os.path.exists(exportPath): # save to backup path
+            exportPath = "%s_1.gpx" % os.path.splitext(logPath)[0]
+          w1.saveToGPX(exportPath)
+          print('tracklog: GPX export successful')
+          # success, delete temporary files
+
+          # primary
+          os.remove(logPath)
+          print('tracklog: temporary file %s deleted' % logPath)
+          # secondary
+          secondaryPath = "%s.temporary_csv_2" % os.path.splitext(logPath)[0]
+          if os.path.exists(secondaryPath):
+            os.remove(secondaryPath)
+            print('tracklog: temporary file %s deleted' % secondaryPath)
+
+        except Exception, e:
+          print('tracklog: exporting unsaved primary log file failed')
+          print(e)
+          failedPath = "%s_1.csv" % os.path.splitext(logPath)[0]
+          print('tracklog: renaming to %s instead' % failedPath)
+          try:
+            shutil.move(logPath, failedPath)
+            print("tracklog: renaming successful")
+          except Exception, e:
+            ('tracklog: renaming %s to %s failed' % (logPath, failedPath))
+            print(e)
+
+
+    # rescan for secondary logs
+    # (there should be only secondary logs that
+    # either don't have primary logs or where primary logs
+    # failed to parse (primary logs delete secondary logs
+    # after successful processing)
+
+    secondaryLogs = glob.glob("%s/*.temporary_csv_2" % logFolder)
+    if secondaryLogs:
+      print('tracklog: exporting %d unsaved secondary log files to GPX' % len(primaryLogs))
+      for logPath in secondaryLogs:
+        # export any found files
+        print('tracklog: exporting %s to GPX' % logPath)
+        try:
+          w2 = way.fromCSV(logPath, delimiter=",")
+          exportPath = "%s.gpx" % os.path.splitext(logPath)[0]
+          # does the GPX file already exist ?
+          # TODO: check if the GPX file is corrupted and swap with newly exported one ?
+          # (eq. caused by a crash during saving the GPX file)
+          if os.path.exists(exportPath): # save to backup path
+            exportPath = "%s_2.gpx" % os.path.splitext(logPath)[0]
+          w2.saveToGPX(exportPath)
+          print('tracklog: GPX export successful')
+          # success, delete temporary file
+
+          # secondary
+          # (primary is either not there or was already removed in primary pass)
+          os.remove(logPath)
+          print('tracklog: temporary file %s deleted' % logPath)
+
+        except Exception, e:
+          print('tracklog: exporting unsaved secondary log file failed')
+          failedPath = "%s_2.csv" % os.path.splitext(logPath)[0]
+          print('tracklog: renaming to %s instead' % failedPath)
+          try:
+            shutil.move(logPath, failedPath)
+            print("tracklog: renaming successful")
+          except Exception, e:
+            ('tracklog: renaming %s to %s failed' % (logPath, failedPath))
+            print(e)
     
   def shutdown(self):
-    # try to save and stop the log
+    # try to stop and save the log
     if self.loggingEnabled:
       self.stopLogging()
 
